//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Realspace/RealSpaceBuilder.h
//! @brief     Defines class RealSpaceBuilder
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_REALSPACE_REALSPACEBUILDER_H
#define BORNAGAIN_GUI_VIEW_REALSPACE_REALSPACEBUILDER_H

#include "GUI/Model/Sample/SampleItem.h"
#include "GUI/View/Realspace/RealSpaceBuilderUtils.h"
#include "GUI/ba3d/view/camera.h"
#include <QVector3D>

class LayerItem;
class MultiLayerItem;
class ParticleLayoutItem;
class Shape3D;
struct SceneGeometry;
class Particle3DContainer;
class ItemWithParticles;

namespace GUI::RealSpace {
class Model;
}

using namespace GUI;

class RealSpaceBuilder {

public:
    //! Defines the item types this class can handle
    using ItemForRealSpace =
        std::variant<MultiLayerItem*, LayerItem*, ParticleLayoutItem*, ItemWithParticles*>;

    RealSpaceBuilder(std::function<QColor(const QString&)> fnColorFromMaterialName);

    void populate(RealSpace::Model* model, ItemForRealSpace item,
                  const SceneGeometry& sceneGeometry,
                  const RealSpace::Camera::Position& cameraPosition =
                      RealSpace::Camera::Position(RealSpace::Vector3D(0, -200, 120), // eye
                                                  RealSpace::Vector3D(0, 0, 0),      // center
                                                  RealSpace::Vector3D::_z))          // up
        const;

    void populateParticleFromParticle3DContainer(RealSpace::Model* model,
                                                 const Particle3DContainer& particle3DContainer,
                                                 const QVector3D& lattice_position = {}) const;

private:
    void populateMultiLayer(RealSpace::Model* model, const MultiLayerItem& mlayerItem,
                            const SceneGeometry& sceneGeometry, const QVector3D& origin = {}) const;

    void populateLayer(RealSpace::Model* model, const LayerItem& layerItem,
                       const SceneGeometry& sceneGeometry, const QVector3D& origin = {},
                       bool isTopLayer = false) const;

    void populateLayout(RealSpace::Model* model, const ParticleLayoutItem& layoutItem,
                        const SceneGeometry& sceneGeometry, const QVector3D& origin = {}) const;

    void populateParticleFromParticleItem(RealSpace::Model* model,
                                          const ItemWithParticles& particleItem) const;


private:
    std::unique_ptr<RealSpace::BuilderUtils> m_builderUtils;
};

#endif // BORNAGAIN_GUI_VIEW_REALSPACE_REALSPACEBUILDER_H
