//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/SampleDesigner/RealSpacePanel.cpp
//! @brief     Implements class RealSpacePanel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/SampleDesigner/RealSpacePanel.h"
#include "GUI/View/Common/StyledToolbar.h"
#include "GUI/View/Realspace/RealSpaceCanvas.h"
#include <QVBoxLayout>

RealSpacePanel::RealSpacePanel(QWidget* parent)
    : QWidget(parent)
    , m_canvas(new RealSpaceCanvas)
{
    setWindowTitle("Real Space");

    auto* toolbar = new StyledToolbar(this);

    const auto createAction = [&](const QString& text, const QString& tooltip) -> QAction* {
        auto* action = new QAction(text, this);
        action->setToolTip(tooltip);
        if (!toolbar->actions().empty())
            toolbar->addSeparator();
        toolbar->addAction(action);
        return action;
    };

    auto* action = createAction("Save Picture", "Save 3D real space view as .png file");
    connect(action, &QAction::triggered, m_canvas, QOverload<>::of(&RealSpaceCanvas::savePicture));

    action = createAction("Default View", "Reset view and zoom level to default");
    connect(action, &QAction::triggered, m_canvas, &RealSpaceCanvas::defaultView);

    action = createAction("Side View", "View sample from the side at current zoom level");
    connect(action, &QAction::triggered, m_canvas, &RealSpaceCanvas::sideView);

    action = createAction("Top View", "View sample from the top at current zoom level");
    connect(action, &QAction::triggered, m_canvas, &RealSpaceCanvas::topView);

    action = createAction("Enlarge", "Increase layer size");
    connect(action, &QAction::triggered, [this]() { m_canvas->changeLayerSize(1.25); });

    action = createAction("Reduce", "Decrease layer size");
    connect(action, &QAction::triggered, [this]() { m_canvas->changeLayerSize(0.8); });

    auto* mainLayout = new QVBoxLayout(this);
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->setSpacing(0);
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->addWidget(toolbar);
    mainLayout->addWidget(m_canvas);
}

QSize RealSpacePanel::sizeHint() const
{
    return QSize(300, 300);
}

RealSpaceCanvas* RealSpacePanel::canvas()
{
    return m_canvas;
}
